@@ -23,30 +23,21 @@
 #include "config.h"
 
 #include "gkr-daemon.h"
+#include "gkr-daemon-util.h"
 
 #include "common/gkr-async.h"
 #include "common/gkr-cleanup.h"
-#include "common/gkr-crypto.h"
-#include "common/gkr-daemon-util.h"
-#include "common/gkr-secure-memory.h"
-#include "common/gkr-unix-credentials.h"
 #include "common/gkr-unix-signal.h"
 
+#include "egg/egg-libgcrypt.h"
+#include "egg/egg-secure-memory.h"
+#include "egg/egg-unix-credentials.h"
+
 #include "keyrings/gkr-keyring-login.h"
 
 #include "library/gnome-keyring.h"
 
-#include "pk/gkr-pk-object-storage.h"
-#ifdef ROOT_CERTIFICATES
-#include "pk/gkr-pk-root-storage.h"
-#endif
-
-#include "pkcs11/gkr-pkcs11-dispatch.h"
-
-#ifdef WITH_SSH
-#include "ssh/gkr-ssh-daemon.h"
-#include "ssh/gkr-ssh-storage.h"
-#endif
+#include "pkcs11/gkr-pkcs11-daemon.h"
 
 #include "ui/gkr-ask-daemon.h"
 
@@ -230,19 +221,19 @@ static gboolean do_warning = TRUE;
  */ 
 
 void
-gkr_memory_lock (void)
+egg_memory_lock (void)
 {
 	/* The daemon uses cooperative threading, and doesn't need locking */
 }
 
 void 
-gkr_memory_unlock (void)
+egg_memory_unlock (void)
 {
 	/* The daemon uses cooperative threading, and doesn't need locking */
 }
 
 void*
-gkr_memory_fallback (void *p, unsigned long sz)
+egg_memory_fallback (void *p, size_t sz)
 {
 	const gchar *env;
 	
@@ -378,7 +369,7 @@ read_login_password (int fd)
 	 * password.
 	 */
 	
-	gchar *buf = gkr_secure_alloc (MAX_BLOCK);
+	gchar *buf = egg_secure_alloc (MAX_BLOCK);
 	gchar *ret = NULL;
 	int r, len = 0;
 	
@@ -387,12 +378,12 @@ read_login_password (int fd)
 		if (r < 0) {
 			if (errno == EAGAIN)
 				continue;
-			gkr_secure_free (ret);
-			gkr_secure_free (buf);
+			egg_secure_free (ret);
+			egg_secure_free (buf);
 			return NULL;
 			
 		} else  { 
-			char *n = gkr_secure_realloc (ret, len + r + 1);
+			char *n = egg_secure_realloc (ret, len + r + 1);
 			memset(n + len, 0, r + 1); 
 			ret = n;
 			len = len + r;
@@ -404,7 +395,7 @@ read_login_password (int fd)
 			break;
 	}
 	
-	gkr_secure_free (buf);
+	egg_secure_free (buf);
 	return ret;
 }
 
@@ -419,7 +410,7 @@ static void
 clear_login_password (void)
 {
 	if(login_password)
-		gkr_secure_strfree (login_password);
+		egg_secure_strfree (login_password);
 	login_password = NULL;
 }
 
@@ -468,20 +459,20 @@ initialize_other_running_daemon (int sock)
 {
 	GnomeKeyringResult res;
 	gchar **envp, **e;
-	GkrBuffer buf;
+	EggBuffer buf;
 	gboolean ret;
 	
-	if (gkr_unix_credentials_write (sock) < 0)
+	if (egg_unix_credentials_write (sock) < 0)
 		return FALSE;
 
-	gkr_buffer_init_full (&buf, 128, (GkrBufferAllocator)g_realloc);
+	egg_buffer_init_full (&buf, 128, g_realloc);
 	
 	envp = gnome_keyring_build_environment (GNOME_KEYRING_IN_ENVIRONMENT);
 	ret = gkr_proto_encode_prepare_environment (&buf, (const gchar**)envp);
 	g_strfreev (envp);
 	
 	if (!ret) {
-		gkr_buffer_uninit (&buf);
+		egg_buffer_uninit (&buf);
 		g_return_val_if_reached (FALSE);
 	}
 
@@ -492,7 +483,7 @@ initialize_other_running_daemon (int sock)
 	      gkr_proto_decode_prepare_environment_reply (&buf, &res, &envp);
 	
 	
-	gkr_buffer_uninit (&buf);
+	egg_buffer_uninit (&buf);
 	
 	if(!ret) {
 		g_warning ("couldn't initialize running daemon");
@@ -634,27 +625,21 @@ gkr_daemon_complete_initialization(void)
 		return TRUE;
 	}
 	
-	/* Initialize object storage */
-	if (!gkr_pk_object_storage_initialize ())
+	/* Initialize new style PKCS#11 components */
+	if (!gkr_pkcs11_daemon_initialize ())
 		return FALSE;
 	
-#ifdef ROOT_CERTIFICATES
-	if (!gkr_pk_root_storage_initialize ())
-		return FALSE;
-#endif
-
 	/* Initialize the appropriate components */
 	
 #ifdef WITH_SSH	
 	if (check_run_component ("ssh")) {
-		if (!gkr_daemon_ssh_io_initialize () ||
-		    !gkr_ssh_storage_initialize ())
+		if (!gkr_pkcs11_daemon_setup_ssh ())
 			return FALSE;
 	}
 #endif
 	
 	if (check_run_component ("pkcs11")) {
-		if (!gkr_pkcs11_dispatch_setup ())
+		if (!gkr_pkcs11_daemon_setup_pkcs11 ())
 			return FALSE;
 	}
 	
@@ -703,8 +688,8 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
 
-	gkr_crypto_setup ();
-
+	egg_libgcrypt_initialize ();
+	
 	/* Send all warning or error messages to syslog */
 	prepare_logging ();
 	
@@ -715,6 +700,11 @@ main (int argc, char *argv[])
 		if (start_or_initialize_daemon ())
 			cleanup_and_exit (0);
 	} 
+
+	/* Initialize our daemon main loop and threading */
+	loop = g_main_loop_new (NULL, FALSE);
+	ctx = g_main_loop_get_context (loop);
+	gkr_async_workers_init (loop);
 	
 	/* 
 	 * Always initialize the keyring subsystem. This is a necessary
@@ -741,9 +731,6 @@ main (int argc, char *argv[])
 	/* Prepare logging a second time, since we may be in a different process */
 	prepare_logging();
 
-	loop = g_main_loop_new (NULL, FALSE);
-	ctx = g_main_loop_get_context (loop);
-	
 	signal (SIGPIPE, SIG_IGN);
 	gkr_unix_signal_connect (ctx, SIGINT, signal_handler, NULL);
 	gkr_unix_signal_connect (ctx, SIGHUP, signal_handler, NULL);
@@ -751,8 +738,6 @@ main (int argc, char *argv[])
              
 	/* TODO: Do we still need this? XFCE still seems to use it. */
 	slave_lifetime_to_fd ();
-	
-	gkr_async_workers_init (loop);
 
 	/*
 	 * Unlock the login keyring if we were given a password on STDIN.
@@ -761,7 +746,7 @@ main (int argc, char *argv[])
 	if (login_password) {
 		if (!gkr_keyring_login_unlock (login_password))
 			g_warning ("Failed to unlock login on startup");
-		gkr_secure_strclear (login_password);
+		egg_secure_strclear (login_password);
 	}
 	
 	g_main_loop_run (loop);
