@@ -28,7 +28,7 @@
 #include "gnome-keyring.h"
 #include "gnome-keyring-opcodes.h"
 
-#include "common/gkr-buffer.h"
+#include "egg/egg-buffer.h"
 
 /* request:
    uint32 package size
@@ -40,186 +40,191 @@
    uint32 result
 */
 
-void     gkr_proto_go_secure                         (GkrBuffer *buffer);
+void     gkr_proto_go_secure                         (EggBuffer *buffer);
 
 /* Core buffer ops */
-gboolean gkr_proto_get_bytes                         (GkrBuffer                  *buffer,
+gboolean gkr_proto_get_bytes                         (EggBuffer                  *buffer,
                                                       gsize                       offset,
                                                       gsize                      *next_offset,
                                                       guchar                     *out,
                                                       gsize                       n_bytes);
-void     gkr_proto_add_time                          (GkrBuffer                  *buffer,
+void     gkr_proto_add_time                          (EggBuffer                  *buffer,
                                                       time_t                      val);
-gboolean gkr_proto_get_time                          (GkrBuffer                  *buffer,
+gboolean gkr_proto_get_time                          (EggBuffer                  *buffer,
                                                       gsize                       offset,
                                                       gsize                      *next_offset,
                                                       time_t                     *time);
-gboolean gkr_proto_add_utf8_string                   (GkrBuffer                  *buffer,
+gboolean gkr_proto_add_utf8_string                   (EggBuffer                  *buffer,
                                                       const char                 *str);
-gboolean gkr_proto_add_utf8_secret                   (GkrBuffer                  *buffer,
+gboolean gkr_proto_add_utf8_secret                   (EggBuffer                  *buffer,
                                                       const char                 *str);
-gboolean gkr_proto_get_utf8_full                     (GkrBuffer                  *buffer,
+gboolean gkr_proto_get_utf8_full                     (EggBuffer                  *buffer,
                                                       gsize                       offset,
                                                       gsize                      *next_offset,
                                                       char                      **str_ret, 
-                                                      GkrBufferAllocator          allocator);
-gboolean gkr_proto_get_utf8_string                   (GkrBuffer                  *buffer,
+                                                      EggBufferAllocator          allocator);
+gboolean gkr_proto_get_utf8_string                   (EggBuffer                  *buffer,
                                                       gsize                       offset,
                                                       gsize                      *next_offset,
                                                       char                      **str_ret);
-gboolean gkr_proto_get_utf8_secret                   (GkrBuffer                  *buffer,
+gboolean gkr_proto_get_utf8_secret                   (EggBuffer                  *buffer,
                                                       gsize                       offset,
                                                       gsize                      *next_offset,
                                                       char                      **str_ret);
-gboolean gkr_proto_add_attribute_list                (GkrBuffer                  *buffer,
+gboolean gkr_proto_add_attribute_list                (EggBuffer                  *buffer,
                                                       GnomeKeyringAttributeList  *attributes);
-gboolean gkr_proto_add_acl                           (GkrBuffer                  *buffer,
+gboolean gkr_proto_add_acl                           (EggBuffer                  *buffer,
                                                       GList                      *acl);
 
 
 /* marshallers */
-gboolean gkr_proto_encode_op_only                    (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_op_only                    (EggBuffer                 *buffer,
                                                       GnomeKeyringOpCode         op);
-gboolean gkr_proto_encode_op_string                  (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_op_string                  (EggBuffer                 *buffer,
                                                       GnomeKeyringOpCode         op,
                                                       const char                *str);
-gboolean gkr_proto_encode_op_string_int              (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_op_string_int              (EggBuffer                 *buffer,
                                                       GnomeKeyringOpCode         op,
                                                       const char                *str,
                                                       guint32                    integer);
-gboolean gkr_proto_encode_op_string_int_int          (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_op_string_int_int          (EggBuffer                 *buffer,
                                                       GnomeKeyringOpCode         op,
                                                       const char                *str,
                                                       guint32                    integer1,
                                                       guint32                    integer2);
-gboolean gkr_proto_encode_op_string_secret           (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_op_string_secret           (EggBuffer                 *buffer,
                                                       GnomeKeyringOpCode         op,
                                                       const char                *str1,
                                                       const char                *str2);
-gboolean gkr_proto_encode_op_string_secret_secret    (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_op_string_secret_secret    (EggBuffer                 *buffer,
                                                       GnomeKeyringOpCode         op,
                                                       const char                *str1,
                                                       const char                *str2,
                                                       const char                *str3);
-gboolean gkr_proto_encode_find                       (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_find                       (EggBuffer                 *buffer,
                                                       GnomeKeyringItemType       type,
                                                       GnomeKeyringAttributeList *attributes);
-gboolean gkr_proto_encode_create_item                (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_create_item                (EggBuffer                 *buffer,
                                                       const char                *keyring,
                                                       const char                *display_name,
                                                       GnomeKeyringAttributeList *attributes,
                                                       const char                *secret,
                                                       GnomeKeyringItemType       type,
                                                       gboolean                   update_if_exists);
-gboolean gkr_proto_encode_set_attributes             (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_set_attributes             (EggBuffer                 *buffer,
                                                       const char                *keyring,
                                                       guint32                    id,
                                                       GnomeKeyringAttributeList *attributes);
-gboolean gkr_proto_encode_set_acl                    (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_set_acl                    (EggBuffer                 *buffer,
                                                       const char                *keyring,
                                                       guint32                    id,
                                                       GList 			*acl);
-gboolean gkr_proto_encode_set_item_info              (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_set_item_info              (EggBuffer                 *buffer,
                                                       const char                *keyring,
                                                       guint32                    id,
                                                       GnomeKeyringItemInfo      *info);
-gboolean gkr_proto_encode_set_keyring_info           (GkrBuffer                 *buffer,
+gboolean gkr_proto_encode_set_keyring_info           (EggBuffer                 *buffer,
                                                       const char                *keyring,
                                                       GnomeKeyringInfo          *info);
+gboolean gkr_proto_encode_prepare_environment        (EggBuffer                 *buffer, 
+                                                      const gchar              **environment);
 
 
 /* demarshallers */
-gboolean gkr_proto_decode_packet_operation           (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_packet_operation           (EggBuffer                  *buffer,
                                                       GnomeKeyringOpCode         *op);
-gboolean gkr_proto_decode_packet_size                (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_packet_size                (EggBuffer                  *buffer,
                                                       guint32                    *size);
-gboolean gkr_proto_decode_attribute_list             (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_attribute_list             (EggBuffer                  *buffer,
                                                       gsize                       offset,
                                                       gsize                      *next_offset,
                                                       GnomeKeyringAttributeList **attributes_out);
-gboolean gkr_proto_decode_acl                        (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_acl                        (EggBuffer                  *buffer,
                                                       gsize                       offset,
                                                       gsize                      *next_offset,
                                                       GList                     **attributes_out);
-gboolean gkr_proto_decode_result_reply               (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_result_reply               (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result);
-gboolean gkr_proto_decode_result_string_reply        (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_result_string_reply        (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result,
                                                       char                      **str);
-gboolean gkr_proto_decode_result_string_list_reply   (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_result_string_list_reply   (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result,
                                                       GList                     **list);
-gboolean gkr_proto_decode_op_string                  (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_op_string                  (EggBuffer                  *buffer,
                                                       GnomeKeyringOpCode         *op_out,
                                                       char                      **str_out);
-gboolean gkr_proto_decode_op_string_secret           (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_op_string_secret           (EggBuffer                  *buffer,
                                                       GnomeKeyringOpCode         *op_out,
                                                       char                      **str1_out,
                                                       char                      **str2_out);
-gboolean gkr_proto_decode_op_string_secret_secret    (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_op_string_secret_secret    (EggBuffer                  *buffer,
                                                       GnomeKeyringOpCode         *op_out,
                                                       char                      **str1_out,
                                                       char                      **str2_out,
                                                       char                      **str3_out);
-gboolean gkr_proto_decode_op_string_int              (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_op_string_int              (EggBuffer                  *buffer,
                                                       GnomeKeyringOpCode         *op_out,
                                                       char                      **str1,
                                                       guint32                    *val);
-gboolean gkr_proto_decode_get_item_info	             (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_get_item_info	             (EggBuffer                  *buffer,
                                                       GnomeKeyringOpCode         *op_out,
                                                       char                      **keyring,
                                                       guint32                    *item_id,
                                                       guint32                    *flags);
-gboolean gkr_proto_decode_find                       (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_find                       (EggBuffer                  *buffer,
                                                       GnomeKeyringItemType       *type,
                                                       GnomeKeyringAttributeList **attributes);
-gboolean gkr_proto_decode_find_reply                 (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_find_reply                 (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result,
                                                       GList                     **list_out);
-gboolean gkr_proto_decode_get_attributes_reply       (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_get_attributes_reply       (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result,
                                                       GnomeKeyringAttributeList **attributes);
-gboolean gkr_proto_decode_get_acl_reply              (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_get_acl_reply              (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result,
                                                       GList                     **acl);
-gboolean gkr_proto_decode_get_item_info_reply        (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_get_item_info_reply        (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result,
                                                       GnomeKeyringItemInfo      **info);
-gboolean gkr_proto_decode_get_keyring_info_reply     (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_get_keyring_info_reply     (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result,
                                                       GnomeKeyringInfo          **info);
-gboolean gkr_proto_decode_result_int_list_reply      (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_result_int_list_reply      (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result,
                                                       GList                     **list);
-gboolean gkr_proto_decode_result_integer_reply       (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_result_integer_reply       (EggBuffer                  *buffer,
                                                       GnomeKeyringResult         *result,
                                                       guint32                    *integer);
-gboolean gkr_proto_decode_create_item                (GkrBuffer                  *packet,
+gboolean gkr_proto_decode_create_item                (EggBuffer                  *packet,
                                                       char                      **keyring,
                                                       char                      **display_name,
                                                       GnomeKeyringAttributeList **attributes,
                                                       char                      **secret,
                                                       GnomeKeyringItemType       *type_out,
                                                       gboolean                   *update_if_exists);
-gboolean gkr_proto_decode_set_item_info              (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_set_item_info              (EggBuffer                  *buffer,
                                                       char                      **keyring,
                                                       guint32                    *item_id,
                                                       GnomeKeyringItemType       *type,
                                                       char                      **display_name,
                                                       char                      **secret);
-gboolean gkr_proto_decode_set_keyring_info           (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_set_keyring_info           (EggBuffer                  *buffer,
                                                       char                      **keyring,
                                                       gboolean                   *lock_on_idle,
                                                       guint32                    *lock_timeout);
-gboolean gkr_proto_decode_set_attributes             (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_set_attributes             (EggBuffer                  *buffer,
                                                       char                      **keyring,
                                                       guint32                    *item_id,
                                                       GnomeKeyringAttributeList **attributes);
-gboolean gkr_proto_decode_set_acl                    (GkrBuffer                  *buffer,
+gboolean gkr_proto_decode_set_acl                    (EggBuffer                  *buffer,
                                                       char                      **keyring,
                                                       guint32                    *item_id,
                                                       GList                     **acl);
-
-
+gboolean gkr_proto_decode_prepare_environment        (EggBuffer                  *buffer, 
+                                                      char                     ***environment);
+gboolean gkr_proto_decode_prepare_environment_reply  (EggBuffer                  *buffer,
+                                                      GnomeKeyringResult         *res,
+                                                      char                     ***environment);
    
 #endif /* GNOME_KEYRING_PROTO_H */
